/* 
 * CUAG AutoUmpire System
 * 
 * Copyright (C) 2003/2004 Matthew Johnson, Adam Biltcliffe,
 * Michael Cripps, Martin O'Leary, Edward Allcutt and James Osborn
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version. 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details. 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * To Contact the authors, please email assassins@srcf.ucam.org
 *
 */

package org.ucam.srcf.assassins;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.ucam.srcf.assassins.exceptions.AssassinSQLException;

/**
 * Viewer for the AssassinState class. 
 */
public class AssassinStateViewer extends Viewer
{

   public AssassinStateViewer(AssassinState as)
   {
      this.ent = as;
   }
   
   /**
    * Determine the assassin this instance represents the state of.
    * @return The assassin object.
    */
   public AssassinViewer getAssassin()
   {
      return new AssassinViewer(((AssassinState)this.ent).getAssassin());
   }
   
   /**
    * Determine whether this assassin is currently alive or dead.
    * @return True if the assassin is alive, false if dead.
    */
   public boolean isAlive()
   {
      return ((AssassinState)this.ent).isAlive();
   }

   /**
    * Determine whether this assassin currently exists in the game.
    * @return True if the assassin is active, false if disabled.
    */
   public boolean isActive()
   {
      return ((AssassinState)this.ent).isActive();
   }
   
   /**
    * Retrieve a List of the pseudonyms currently attached to this
    * assassin.
    * @return a {@link java.util.List} of strings representing pseudonyms
    *    currently being used by this assassin.
    */
   public List getPseudonyms() throws AssassinSQLException
   {
      List src = ((AssassinState)this.ent).getPseudonyms();
      List res = new ArrayList(src.size());
      Iterator i = src.iterator();
      while (i.hasNext())
      {
         Pseudonym p = (Pseudonym) i.next();
         res.add(p.getValue());
      }
      return res;
   }

   /**
    * Retrieve a specific element of this player's current plugin-specific
    * state, identified by the name of the plugin in question and the field
    * name.
    * @param pname The name of the plugin for which state is being retrieved.
    * @param field The name of the field whose value is being retrieved.
    * @return The value of the field.
    */
   public String getProperty(String pname, String field)
   {
      return (String) ((AssassinState)this.ent).getProperty(pname, field);
   }

   public int getIntProperty(String pname, String field)
   {
      return ((AssassinState)this.ent).getIntProperty(pname, field);
   }

   public boolean getBoolProperty(String pname, String field)
   {
      if (Debug.debug) Debug.print(AssassinStateViewer.class, "in getBoolProperty");
      boolean b = ((AssassinState)this.ent).getBoolProperty(pname, field);
      if (Debug.debug) Debug.print(AssassinStateViewer.class, "returning");
      return b;
   }

   public String getProperty(Plugin p, String field)
   {
      return this.getProperty(p.getPluginName(), field);
   }

   public int getIntProperty(Plugin p, String field)
   {
      return this.getIntProperty(p.getPluginName(), field);
   }

   public boolean getBoolProperty(Plugin p, String field)
   {
      return this.getBoolProperty(p.getPluginName(), field);
   }
}
