/* 
 * CUAG AutoUmpire System
 * 
 * Copyright (C) 2003/2004 Matthew Johnson, Adam Biltcliffe,
 * Michael Cripps, Martin O'Leary, Edward Allcutt and James Osborn
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version. 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details. 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * To Contact the authors, please email assassins@srcf.ucam.org
 *
 */

package org.ucam.srcf.assassins;

import java.io.IOException;
import java.text.ParseException;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Vector;

import cx.ath.matthew.cgi.CGI;
import cx.ath.matthew.cgi.CGIHeaderSentException;
import cx.ath.matthew.cgi.HiddenField;
import cx.ath.matthew.cgi.TextField;

import org.ucam.srcf.assassins.exceptions.AssassinSQLException;
import org.ucam.srcf.assassins.exceptions.StateBuilderException;

public class CGIEditPseudonym implements CGICommand
{
   private void editPseudonym(CGI cgi, Map plugins, Map params) throws AssassinSQLException, IOException, ParseException, StateBuilderException
   { 
      Debug.printMap(umpirecgi.class, params);
      if ("2".equals(params.get("pass")) || (null != params.get("1") && !"".equals(params.get("1")))) {
         HTMLForm form = new HTMLForm("umpire.cgi?edit-pseudonym", "Edit");
         Pseudonym p = (Pseudonym) params.get("pseudonym");
         if (null == p) p = Pseudonym.getByPsid(Integer.parseInt((String) params.get("1")));
         if (Debug.debug) Debug.print(umpirecgi.class, "pseudonym: "+p);
         cgi.out("<h2>Edit Pseudonym</h2>");
         form.addField(new HiddenField("pass","3"));
         form.addField(new HiddenField("pseudonym",""+p));
         form.addField(new TextField("value","Pseudonym:",Tools.escapeXML(p.getValue())));
         form.display(cgi);
      } else if ("3".equals(params.get("pass"))) {
         Pseudonym pseudonym = (Pseudonym) params.get("pseudonym");
         if (!umpirecgi.currentAuth.isAtLeast(AuthLevel.UMPIRE)) {
            Assassin a = pseudonym.getAssassin();
            if (!a.getPlayer().equals(umpirecgi.currentAuth.getPlayerViewer().getEntity())) {
               cgi.out("<div class=\"errormessage\">");
               cgi.out("   <p><b>Insufficient Privileges</b></p>");
               cgi.out("   <p>I'm sorry, only the umpire can change other people's pseudonyms.</p>");
               cgi.out("</div>");
               return;
            }
         }
         String value = (String) params.get("value");
         pseudonym.setValue(value);
         Pseudonym.commitAll();
         try { cgi.header("Refresh", "2; URL=umpire.cgi"); } catch (CGIHeaderSentException CGIHSe) {} 
         cgi.out("<h2>Edit Pseudonym</h2>");
         cgi.out("<div class=\"donemessage\">");
         cgi.out("  <p><b>Pseudonym Updated</b></p>");
         cgi.out("  <p>&nbsp;</p>");
         cgi.out("</div>");
      } else {
         HTMLForm form = new HTMLForm("umpire.cgi?edit-pseudonym", "Edit");
         cgi.out("<h2>Edit Pseudonym</h2>");
         cgi.out("<p>");
         cgi.out("  Select Pseudonym to edit:");
         cgi.out("</p>");
         //Event.getAllEventsWithHeadline();
         if (umpirecgi.currentAuth.isAtLeast(AuthLevel.UMPIRE))
            form.addField(new CanonicalEntityField("pseudonym","Pseudonym:",Pseudonym.getAllPseudonyms(),(CanonicalEntity) null));
         else {
            Vector v = new Vector();
            List u = ((Player) umpirecgi.currentAuth.getPlayerViewer().getEntity()).getAssassins();
            Iterator i = u.iterator();
            LightweightStateBuilder lwsb = new LightweightStateBuilder(Tools.getNowTime());
            while (i.hasNext()) {
               Assassin a = (Assassin) i.next();
               lwsb.registerInterest(a);
            }
            Map m = lwsb.generateState();
            i = m.values().iterator();
            while (i.hasNext()) {
               AssassinState as = (AssassinState) i.next();
               v.addAll(as.getPseudonyms());
            }
            form.addField(new CanonicalEntityField("pseudonym","Pseudonym:",v,(CanonicalEntity) null));
         }
         form.addField(new HiddenField("pass","2"));
         form.display(cgi);
      }
   }

   public void command(CGI cgi, Map plugins, Map params, long nowtime) 
      throws AssassinSQLException, IOException, ParseException, StateBuilderException {  editPseudonym(cgi, plugins, params); } 
   public String getName() { return "Edit Pseudonym"; }
   public String getDescription() { return "Change an existing pseudonym"; }
   public AuthLevel accessLevel () { return AuthLevel.PLAYER; }
   public String getSection() { return "Assassin"; }
}

