/*
 * Created on 05-Nov-2004
 *
 */
package org.ucam.srcf.assassins;

import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.ucam.srcf.assassins.exceptions.AssassinSQLException;
import org.ucam.srcf.assassins.exceptions.StateBuilderException;

/**
 * @author Mike Cripps
 * 
 */
public class LightweightStateBuilder {

	private List assassins;
	private long time;

	/**
	 * 
	 */
	public LightweightStateBuilder(long time) {
		this.assassins = new LinkedList();
		this.time = time;
	}

	/**
	 * @param a
	 */
	public void registerInterest(Assassin a) {
		this.assassins.add(a);
	}

	public List getInvolved() {
		return assassins;
	}

	/**
	 * @return
	 * @throws StateBuilderException
	 */
	public Map generateState() throws StateBuilderException {
		// Generate state only for assassins in the List 'assassins'
		// Should be much faster! ;)
		// Mike
		if (Debug.debug) Debug.print(this, "Generating State");
		Map states = new HashMap();
		Iterator i = assassins.iterator();
		while (i.hasNext()) {
			Assassin a = (Assassin) i.next();
			try {
				if (Debug.debug) Debug.print(this, "Generating State for "+ a.getPlayer().getName());
			} catch (AssassinSQLException ex) {
			}
			AssassinState s = new AssassinState(a);
			states.put(a, s);
			try {
				List l = StateChange.getActiveByAssassin(a, time);
				Iterator li = l.iterator();
				while (li.hasNext()) {
					StateChange sc = (StateChange) li.next();
					String pname = sc.getPluginName();
					String field = sc.getFieldName();
					Object value = sc.getValue();
					if (Debug.debug) Debug.print(this, "Implementing:" + sc + " / " + value	+ " (" + pname + ")");
					if (sc instanceof PseudonymGain) {
						if (Debug.debug) Debug.print(this, "Adding Pseudonym \""+ ((Pseudonym) value).getValue() + "\"");
						s.addPseudonym((Pseudonym) value);
					} else if (sc instanceof PseudonymLoss) {
						if (Debug.debug) Debug.print(this, "Losing Pseudonym \""	+ ((Pseudonym) value).getValue() + "\"");
						s.removePseudonym((Pseudonym) value);
					} else {
						if (sc instanceof Death) {
							if (Debug.debug) Debug.print(this, "Died");
							field = "alive";
							value = "false";
						} else if (sc instanceof Resurrection) {
							if (Debug.debug) Debug.print(this, "Rose from the Dead");
							field = "alive";
							value = "true";
						}
						s.setProperty(pname, field, value);
					}
				}
			} catch (AssassinSQLException asqle) {
				throw new StateBuilderException(asqle);
			}

		}

		return states;
	}

}
