/* 
 * CUAG AutoUmpire System
 * 
 * Copyright (C) 2003/2004 Matthew Johnson, Adam Biltcliffe,
 * Michael Cripps, Martin O'Leary, Edward Allcutt and James Osborn
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version. 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details. 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * To Contact the authors, please email assassins@srcf.ucam.org
 *
 */

package org.ucam.srcf.assassins;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.util.List;
import java.util.Vector;

/**
 * Public Superclass for queries plugins can ask the user. 
 *
 * @author Matthew Johnson <autoumpire@matthew.ath.cx>
 */
public abstract class Question
{
   String label;
   CanonicalViewer hidden = null;
   public Question(String label)
   {
      this.label = label;
   }
   public CanonicalViewer getHidden() {
	   return hidden;
   }
   public void setHidden(CanonicalViewer h) {
	   hidden = h;
   }
   public String getLabel()
   {
      return label;
   }
   public abstract String serialize();
   public static Question deSerialise(String input)
   {
      try {
         int i = input.indexOf('(');
         String classname = input.substring(0, i);
         String params = input.substring(i+1, input.length()-1);
         i = params.indexOf(',');
         String label = params.substring(0, i);
         int j = params.indexOf(',', i+1);
         String hidstr = params.substring(i+1, j);
         String remain = params.substring(j+1);
         Class qclass = Class.forName(classname);
         Question q = null;
         CanonicalViewer hidden = null;
         try {
            CanonicalEntity ce = CanonicalEntity.deSerialise(hidstr);
            hidden = (CanonicalViewer) ce.getViewer();
         } catch(Exception e) {}         

         if (qclass.equals(TextQuestion.class)) {
            Constructor qcons = qclass.getConstructor(new Class[] { String.class, String.class });
            q = (Question) qcons.newInstance(new Object[] { label, remain });
            q.setHidden(hidden);
         } else if (qclass.equals(YesNoQuestion.class)) {
            Constructor qcons = qclass.getConstructor(new Class[] { String.class, Boolean.TYPE });
            q = (Question) qcons.newInstance(new Object[] { label, Boolean.valueOf(remain) });
            q.setHidden(hidden);
         } else if (qclass.equals(DisplayData.class)) {
            Constructor qcons = qclass.getConstructor(new Class[] { String.class });
            q = (Question) qcons.newInstance(new Object[] { remain });
            q.setHidden(hidden);
         } else if (qclass.equals(OptionQuestion.class)) {
            i = remain.indexOf(',');
            String defstr = remain.substring(0,i);
            List options = new Vector();
            i++;
            j = remain.indexOf(';', i);
            while(j > 0) {
               options.add(remain.substring(i, j));
               i = j+1;
               j = remain.indexOf(';', i);
            }
            Constructor qcons = qclass.getConstructor(new Class[] { String.class, List.class, Integer.TYPE });
            q = (Question) qcons.newInstance(new Object[] { label, options, new Integer(defstr) });
            q.setHidden(hidden);
         }
         return q;
      } catch (ClassNotFoundException CNFe) {
         return null;
      } catch (NoSuchMethodException NSMe) {
         return null;
      } catch (InstantiationException Ie) {
         return null;
      } catch (IllegalAccessException IAe) {
         return null;
      } catch (InvocationTargetException ITe) {
         return null;
      }
   }
}

