package org.ucam.srcf.assassins.xml;

import java.io.OutputStream;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.Stack;
import java.util.TreeSet;

import org.ucam.srcf.assassins.Assassin;
import org.ucam.srcf.assassins.AssassinState;
import org.ucam.srcf.assassins.AssassinStateViewer;
import org.ucam.srcf.assassins.NewsPage;
import org.ucam.srcf.assassins.Plugin;
import org.ucam.srcf.assassins.Tools;
import org.ucam.srcf.assassins.exceptions.AssassinSQLException;
import org.xml.sax.Attributes;

public class PageHandler extends Handler {

	public PageHandler(OutputStream o,String t,Map s,Map p,Set s2) {
		super(o,t,s,p,s2);
	}
	
	public void startDocument() {
		out.println("<?xml version=\"1.0\" encoding=\"UTF-8\" ?>");
		out.println("<webpage>");
	}
	public void endDocument() {
		out.println("  <links>");
		if (linkedPages!=null) {
			for (Iterator i = linkedPages.iterator(); i.hasNext();) {
			NewsPage np = (NewsPage) i.next();
			String l = np.getLinkName();
			String t = np.getTitle();
			if (l!=target) out.println(" <a href=\""+l+"\">"+t+"</a>");
		}
		}
		out.println("    <a href=\""
				+ Config.getConfig("vanilla", "updates-page-name")
				+ ".html\">Updates</a>");
		out.println("    <a href=\""
				+ Config.getConfig("vanilla", "headlines-page-name")
				+ ".html\">Headlines</a>");
		int index = 1;
		String s = null;
		while (null != (s = Config.getConfig("vanilla", "template-page-name-"
				+ index))) {
			out.println("    <a href=\"" + s + ".html\">" + s + "</a>");
			index++;
		}
		out.println("  </links>");
		out.println("</webpage>");
		out.close();
	}

	public void startElement(String namespaceURI, String name, String qName, Attributes attributes) {
		if (name.equalsIgnoreCase("title")) {
			out.print("<title>");
			currentState = Handler.TEXT;
			return;
		}
		if (name.equalsIgnoreCase("text")) {
			out.print("<text>");
			currentState = Handler.TEXT;
			return;
		}
		if (name.equalsIgnoreCase("empty")) {
			if (currentList.size() == 0) {
				out.print("<text>");
			}
			currentState = PageHandler.EMPTY;
			return;
		}
		if (name.equalsIgnoreCase("list")) {
			currentList = new LinkedList(state.keySet());
			toRemove = new HashSet();
			columns = new HashMap();
			columnList = new LinkedList();
			sorter=null;
			pendingFilters=new Stack();
			return;
		}
		if (name.equalsIgnoreCase("sort")) {
			sorter = new AssComp(attributes.getValue("plugin"),
					attributes.getValue("name"),
					state,
					new Boolean(attributes.getValue("descending")).booleanValue(),
					new Boolean(attributes.getValue("numeric")).booleanValue());
			return;
		}
		if (name.equalsIgnoreCase("column")) {
			List temp = new LinkedList();
			temp.add(attributes.getValue("name"));
			temp.add(attributes.getValue("plugin"));
			columns.put(attributes.getValue("title"), temp);
			columnList.add(attributes.getValue("title"));
			return;
		}
		if (name.equalsIgnoreCase("and")) {
			pendingFilters.push(filters);
			filters = new Stack();
			return;
		}
		if (name.equalsIgnoreCase("filter")) {
			String nm = attributes.getValue("name");
			currentColourFilter = nm;
			if (nm==null) {
				List temp = new LinkedList();
				temp.add(attributes.getValue("property"));
				temp.add(attributes.getValue("condition"));
				temp.add(attributes.getValue("plugin"));
				filters.add(temp);
			}
			return;
		}
		if (name.equalsIgnoreCase("or")) {
			pendingFilters.push(filters);
			filters = new Stack();
			return;
		}
		if (name.equalsIgnoreCase("webpage")) return;
		if (currentState==PageHandler.EMPTY && currentList.size()>0) return;
		out.print("<"+name+">");
	}

	public void endElement(String namespaceURI, String name, String qName) {
		if (name.equalsIgnoreCase("title")) {
			out.println("</title>");
			currentState=Handler.NOTHING;
			return;
		}
		if (name.equalsIgnoreCase("text")) {
			out.println("</text>");
			currentState=Handler.NOTHING;
			return;
		}
		if (name.equalsIgnoreCase("empty")) {
			if (currentList.size()==0) out.println("</text>");
			currentState=Handler.NOTHING;
			return;
		}
		if (name.equalsIgnoreCase("list")) {
			currentList.removeAll(toRemove);
			Set sortSet;
			if (sorter!=null) sortSet = new TreeSet(sorter); else sortSet = new HashSet();
			sortSet.addAll(currentList);
			if (currentList.size()>0) {
				out.println("<table class=\"playerlist\">");
				out.print("  <tr>");
				Iterator i = columnList.iterator();
				while (i.hasNext()) {
					String s = (String) i.next();
					out.print("<td>"+Tools.escapeXML(s)+"</td>");
				}
				out.println("</tr>");
				i = sortSet.iterator();
				while (i.hasNext()) {
					Assassin a = (Assassin) i.next();
					out.print("<tr>");
					Iterator j = columnList.iterator();
					while (j.hasNext()) {
						out.print("<td>");
						List l = (List)columns.get(j.next());
						String nm = (String) l.get(0);
						String pl = (String) l.get(1);
						if (null==pl || "".equals(pl)) pl="vanilla";
						String s = (String) ((AssassinState)state.get(a)).getProperty(pl, nm);
						out.print(Tools.obfuscateEmail(Tools.escapeXML(s)));
						out.print("</td>");
					}
					out.println("</tr>");
				}
				out.println("</table>");
			}
			currentList=null;
			columns=null;
			sorter=null;
			toRemove=null;
			return;
		}
		if (name.equalsIgnoreCase("and")) {
			Iterator i = currentList.iterator();
			while (i.hasNext()) {
				Iterator j = filters.iterator();
				Assassin a = (Assassin)i.next();
				boolean b=true;
				while (j.hasNext()) {
					List l = (List)j.next();
					String property = (String) l.get(0);
					String condition = (String) l.get(1);
					String pl = (String) l.get(2);
					if (null==pl || "".equals(pl)) pl="vanilla";
			        Plugin p = (Plugin) plugins.get(pl);
					try {
				        b &= p.assassinHasProperty(new AssassinStateViewer((AssassinState)state.get(a)), property,condition);
					} catch (AssassinSQLException asqle) {
						b=false;
			        }
				}
				if (!b) toRemove.add(a);
			}
			filters=(List)pendingFilters.pop();
			return;
		}
		if (name.equalsIgnoreCase("or")) {
			Iterator i = currentList.iterator();
			while (i.hasNext()) {
				Iterator j = filters.iterator();
				Assassin a = (Assassin)i.next();
				boolean b=false;
				while (j.hasNext()) {
					List l = (List)j.next();
					String property = (String) l.get(0);
					String condition = (String) l.get(1);
					String pl = (String) l.get(2);
					if (null==pl || "".equals(pl)) pl="vanilla";
					Plugin p = (Plugin) plugins.get(pl);
					try {
						b |= p.assassinHasProperty(new AssassinStateViewer((AssassinState)state.get(a)), property,condition);
					} catch (AssassinSQLException asqle) {
						b=false;
					}
				}
				if (!b) toRemove.add(a);
			}
			filters=(List)pendingFilters.pop();
			return;
		}
		if (name.equalsIgnoreCase("column")) return;
		if (name.equalsIgnoreCase("filter")) {
			if (currentColourFilter==null) return;
			ColourFilter cf = (ColourFilter) Config.getColourFilters().get(currentColourFilter);
			Iterator i = currentList.iterator();
			while (i.hasNext()) {
				Assassin a = (Assassin) i.next();
				AssassinState as = (AssassinState) state.get(a);
				boolean b = cf.matchAssassinState(plugins, as);
				if (!b) toRemove.add(a);
			}
			currentColourFilter=null;
			return;
		}
		if (name.equalsIgnoreCase("sort")) return;
		if (name.equalsIgnoreCase("webpage")) return;
		if (currentState==Handler.EMPTY && currentList.size()>0) return;
		out.print("</"+name+">");
	}
	
}